#!/usr/bin/python
# -*- coding: utf-8 -*-
# 
#   FILE: Elect2012TweetsDB.py
#   DATE: December, 2012
#   Author: David W. McDonald
#
#   An object for managing the fitness tweet collection
#
#   Copyright by Author. All rights reserved. Not for reuse without
#   express permissions.
#
#
from hcde.data.db.base.TweetsDB import TweetsDB
from hcde.data.db.elect2012.Elect2012TweetObj import Elect2012TweetObj
from hcde.data.db.elect2012.Elect2012UserObj import Elect2012UserObj
from hcde.data.db.elect2012.Elect2012UserMetaObj import Elect2012UserMetaObj
from hcde.data.db.elect2012.Elect2012FollowerObj import Elect2012FollowerObj
from hcde.data.db.elect2012.Elect2012FriendObj import Elect2012FriendObj
from sqlalchemy.orm import mapper, class_mapper, clear_mappers
from sqlalchemy.orm.exc import UnmappedClassError
import sys

class Elect2012TweetsDB(TweetsDB):
    def __init__(self, config=None):
        TweetsDB.__init__(self, config=config)

        # Seems to fix a intermittent 'mapper exception'
        # NOTE: this probably prevents having two DBs 'open' at the same time
        clear_mappers()
        try:
            self.tweet_mapper = class_mapper(Elect2012TweetObj)
        except UnmappedClassError:
            self.tweet_mapper = mapper(Elect2012TweetObj,self.tweet_table)

        try:
            self.user_mapper = class_mapper(Elect2012UserObj)
        except UnmappedClassError:
            self.user_mapper = mapper(Elect2012UserObj,self.user_table)

        try:
            self.user_meta_mapper = class_mapper(Elect2012UserMetaObj)
        except UnmappedClassError:
            self.user_meta_mapper = mapper(Elect2012UserMetaObj,self.user_meta_table)

        try:
            self.friend_mapper = class_mapper(Elect2012FriendObj)
        except UnmappedClassError:
            self.friend_mapper = mapper(Elect2012FriendObj,self.friends_table)

        try:
            self.follower_mapper = class_mapper(Elect2012FollowerObj)
        except UnmappedClassError:
            self.follower_mapper = mapper(Elect2012FollowerObj,self.followers_table)


##
# New object creation routines
##

    ## Create "tweet" objects and records
    def new_tweet_table_item(self, rec=None):
        nto = Elect2012TweetObj()
        if( rec ):
            return nto.from_dict(rec)
        return nto
        
    def tweet_table_item_to_dict(self, tto=None):
        rec = {}
        if( tto ):
            rec = tto.to_dict()
        return rec

    ## Create "user" objects and records
    def new_user_table_item(self, rec=None):
        uto = Elect2012UserObj()
        if( rec ):
            return uto.from_dict(rec)
        return uto
        
    def user_table_item_to_dict(self, uto=None):
        rec = {}
        if( uto ):
            rec = uto.to_dict()
        return rec

    ## Create "user_meta" objects and records
    def new_user_meta_table_item(self, rec=None):
        umto = Elect2012UserMetaObj()
        if( rec ):
            return umto.from_dict(rec)
        return umto
        
    def user_meta_table_item_to_dict(self, umto=None):
        rec = {}
        if( umto ):
            rec = umto.to_dict()
        return rec

    ## Create "friend" objects and records
    def new_friend_table_item(self, rec=None):
        nf = Elect2012FriendObj()
        if( rec ):
            return nf.from_dict(rec)
        return nf
        
    def friend_table_item_to_dict(self, nf=None):
        rec = {}
        if( nf ):
            rec = nf.to_dict()
        return rec

    ## Create "follower" objects and records
    def new_follower_table_item(self, rec=None):
        nf = Elect2012FollowerObj()
        if( rec ):
            return nf.from_dict(rec)
        return nf
        
    def follower_table_item_to_dict(self, nf=None):
        rec = {}
        if( nf ):
            rec = nf.to_dict()
        return rec

##
# straight forward query types
##
    ##
    ## Query the tweet table
    ##
    def query_tweet_table_by_tweet_id(self, tid):
        t = self.session.query(Elect2012TweetObj).filter(Elect2012TweetObj.tweet_id==tid).all()
        return t

    def query_tweet_table_by_username(self, uname=None, start_date=None, end_date=None):
        q = self._tweet_table_date_range(start_date=start_date, end_date=end_date)
        q = q.filter(Elect2012TweetObj.from_user==uname)
        tlist = q.all()
        return tlist

    def query_tweet_table_by_user_id(self, uid=None, start_date=None, end_date=None):
        q = self._tweet_table_date_range(start_date=start_date, end_date=end_date)
        q = q.filter(Elect2012TweetObj.from_user_id==uid)
        tlist = q.all()
        return tlist

    def query_tweet_table_by_tweet_substr(self, substr=None, start_date=None, end_date=None):
        qtext = "%"+substr+"%"
        q = self._tweet_table_date_range(start_date=start_date, end_date=end_date)
        q = q.filter(Elect2012TweetObj.tweet_text.like(qtext))
        tlist = q.all()
        return tlist

    def query_tweet_table_by_date_range(self, start_date=None, end_date=None, in_order=True):
        q = self._tweet_table_date_range(start_date=start_date, end_date=end_date)
        if( in_order ):
            q = q.order_by(Elect2012TweetObj.created_at)
        tlist = q.all()
        return tlist

    def _tweet_table_date_range(self, start_date=None, end_date=None):
        query = self.session.query(Elect2012TweetObj)
        if( start_date and end_date ):
            query = query.filter(Elect2012TweetObj.created_at>=start_date)
            query = query.filter(Elect2012TweetObj.created_at<end_date)
        elif( start_date ):
            query = query.filter(Elect2012TweetObj.created_at>=start_date)
        elif( end_date ):
            query = query.filter(Elect2012TweetObj.created_at<end_date)
        else:
            pass
        return query


    ##
    ## Query the user table
    ##
    def query_user_table_by_record_id(self, rid=None, rid2=None):
        if( rid2 ):
            q = self.session.query(Elect2012UserObj).filter(Elect2012UserObj.rid>=rid)
            tlist = q.filter(Elect2012UserObj.rid<rid2).all()
        else:
            tlist = self.session.query(Elect2012UserObj).filter(Elect2012UserObj.rid==rid).all()
        return tlist
        
    def query_user_table_by_fullname(self, sname):
        tlist = self.session.query(Elect2012UserObj).filter(Elect2012UserObj.screen_name==sname).all()
        return tlist

    def query_user_table_by_screenname(self, sname):
        tlist = self.session.query(Elect2012UserObj).filter(Elect2012UserObj.screen_name==sname).all()
        return tlist

    def query_user_table_by_username(self, uname):
        tlist = self.session.query(Elect2012UserObj).filter(Elect2012UserObj.user_name==uname).all()
        return tlist

    def query_user_table_by_user_id(self, uid):
        tlist = self.session.query(Elect2012UserObj).filter(Elect2012UserObj.user_id==uid).all()
        return tlist


    ##
    ## Query the user_meta table
    ##
    def query_user_meta_table_by_record_id(self, rid=None, rid2=None):
        if( rid2 ):
            q = self.session.query(Elect2012UserMetaObj).filter(Elect2012UserMetaObj.rid>=rid)
            tlist = q.filter(Elect2012UserMetaObj.rid<rid2).all()
        else:
            tlist = self.session.query(Elect2012UserMetaObj).filter(Elect2012UserMetaObj.rid==rid).all()
        return tlist
        
    def query_user_meta_table_by_fullname(self, sname):
        tlist = self.session.query(Elect2012UserMetaObj).filter(Elect2012UserMetaObj.screen_name==sname).all()
        return tlist

    def query_user_meta_table_by_screenname(self, sname):
        tlist = self.session.query(Elect2012UserMetaObj).filter(Elect2012UserMetaObj.screen_name==sname).all()
        return tlist

    def query_user_meta_table_by_username(self, uname):
        tlist = self.session.query(Elect2012UserMetaObj).filter(Elect2012UserMetaObj.user_name==uname).all()
        return tlist

    def query_user_meta_table_by_user_id(self, uid):
        tlist = self.session.query(Elect2012UserMetaObj).filter(Elect2012UserMetaObj.user_id==uid).all()
        return tlist


    ##
    ## Query the friends table
    ##
    def query_friends_by_username(self, uname=None, fname=None):
        q = self.session.query(Elect2012FriendObj).filter(Elect2012FriendObj.user==uname)
        if( fname ):
            q = q.filter(Elect2012FriendObj.friend==fname)
        flist = q.all()
        return flist

    def query_friends_by_user_id(self, uid=None, fid=None):
        q = self.session.query(Elect2012FriendObj).filter(Elect2012FriendObj.user_id==uid)
        if( fid ):
            q = q.filter(Elect2012FriendObj.friend_id==fid)
        flist = q.all()
        return flist


    ##
    ## Query the followers table
    ##
    def query_followers_by_username(self, uname=None, fname=None):
        q = self.session.query(Elect2012FollowerObj).filter(Elect2012FollowerObj.user==uname)
        if( fname ):
            q = q.filter(Elect2012FollowerObj.follower==fname)
        flist = q.all()
        return flist

    def query_followers_by_user_id(self, uid=None, fid=None):
        q = self.session.query(Elect2012FollowerObj).filter(Elect2012FollowerObj.user_id==uid)
        if( fid ):
            q = q.filter(Elect2012FollowerObj.follower_id==fid)
        flist = q.all()
        return flist



